import 'dart:typed_data';
import 'package:asn1lib/asn1lib.dart';
import 'package:encrypt/encrypt.dart' as encry;
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:flutter_getitpayment/src/card/enter_card.dart';
import 'package:flutter_getitpayment/src/models/payment_trans_model.dart';
import 'package:flutter_getitpayment/src/models/server_response.dart';
import 'package:pointycastle/asymmetric/api.dart';
import 'dart:convert';
import 'dart:developer';
import 'package:http/http.dart' as http;

///Getit Payments is a src gateway for businesses
class GetItPayments{
  /// Find your company id by logging into GetItPayments.com and going to settings.
static String _companyId = "";
static String _publicKeyString = "";
static bool _testMode = true;

///initialize the pluging by entering your id and public key.
/// For 'publicKey' enter the entire key, even -----BEGIN RSA PUBLIC KEY-----
static initialize({required String companyID,
  required String publicKeySTRING, required bool setTestMode}){
  GetItPayments._companyId = companyID;
  GetItPayments._publicKeyString = publicKeySTRING;
  GetItPayments._testMode = setTestMode;
  debugPrint("Getit payments initialized");
}

 bool get testMode => _testMode;

/// launch the card page to get card information.
/// ONLY If status = OK and message= APPROVED then the money was transfered
Future<ServerResponse?> startTransaction({
  required double amountToPay, required String currency, required BuildContext context,
})async{
  if(_companyId.isEmpty || _publicKeyString.isEmpty){
    const snackBar = SnackBar(
        backgroundColor: Colors.white,
        content: Text(
          'Getit Payments Not Initialized.',
          style: TextStyle(color: Colors.black),
        ));
    ScaffoldMessenger.of(context).showSnackBar(snackBar);
    return null;
  }else{
    return await Navigator.of(context).push(PageRouteBuilder(
        pageBuilder: (_, __, ___) => new EnterCard(
          amountToPay: amountToPay,
          currency: currency,
        ),
        transitionDuration: Duration(milliseconds: 600),
        transitionsBuilder:
            (_, Animation<double> animation, __, Widget child) {
          return Opacity(
            opacity: animation.value,
            child: child,
          );
        }));
  }

}

List<int> _decodePEM(String pem) {
    return base64.decode(removePemHeaderAndFooter(pem));
  }
  String removePemHeaderAndFooter(String pem) {
    var startsWith = [
      "-----BEGIN PUBLIC KEY-----",
      "-----BEGIN RSA PRIVATE KEY-----",
      "-----BEGIN RSA PUBLIC KEY-----",
      "-----BEGIN PRIVATE KEY-----",
      "-----BEGIN PGP PUBLIC KEY BLOCK-----\r\nVersion: React-Native-OpenPGP.js 0.1\r\nComment: http://openpgpjs.org\r\n\r\n",
      "-----BEGIN PGP PRIVATE KEY BLOCK-----\r\nVersion: React-Native-OpenPGP.js 0.1\r\nComment: http://openpgpjs.org\r\n\r\n",
    ];
    var endsWith = [
      "-----END PUBLIC KEY-----",
      "-----END PRIVATE KEY-----",
      "-----END RSA PRIVATE KEY-----",
      "-----END RSA PUBLIC KEY-----",
      "-----END PGP PUBLIC KEY BLOCK-----",
      "-----END PGP PRIVATE KEY BLOCK-----",
    ];
    bool isOpenPgp = pem.indexOf('BEGIN PGP') != -1;

    pem = pem.replaceAll(' ', '');
    pem = pem.replaceAll('\n', '');
    pem = pem.replaceAll('\r', '');

    for (var s in startsWith) {
      s = s.replaceAll(' ', '');
      if (pem.startsWith(s)) {
        pem = pem.substring(s.length);
      }
    }

    for (var s in endsWith) {
      s = s.replaceAll(' ', '');
      if (pem.endsWith(s)) {
        pem = pem.substring(0, pem.length - s.length);
      }
    }

    if (isOpenPgp) {
      var index = pem.indexOf('\r\n');
      pem = pem.substring(0, index);
    }
    return pem;
  }

  RSAPublicKey _parsePublicKeyFromPem(pemString) {
    List<int> publicKeyDER = _decodePEM(pemString);
    ASN1Parser asn1Parser = new ASN1Parser(publicKeyDER as Uint8List);
    ASN1Sequence topLevelSeq = asn1Parser.nextObject() as ASN1Sequence;

    var modulus, exponent;
    // Depending on the first element type, we either have PKCS1 or 2
    if (topLevelSeq.elements[0].runtimeType == ASN1Integer) {
      modulus = topLevelSeq.elements[0] as ASN1Integer;
      exponent = topLevelSeq.elements[1] as ASN1Integer;
    } else {
      var publicKeyBitString = topLevelSeq.elements[1];

      var publicKeyAsn = new ASN1Parser(publicKeyBitString.contentBytes()!);
      ASN1Sequence publicKeySeq = publicKeyAsn.nextObject() as ASN1Sequence;
      modulus = publicKeySeq.elements[0] as ASN1Integer;
      exponent = publicKeySeq.elements[1] as ASN1Integer;
    }

    RSAPublicKey rsaPublicKey =
    RSAPublicKey(modulus.valueAsBigInteger, exponent.valueAsBigInteger);

    return rsaPublicKey;
  }

  RSAPrivateKey _parsePrivateKeyFromPem(pemString) {
    List<int> privateKeyDER = _decodePEM(pemString);
    var asn1Parser = new ASN1Parser(privateKeyDER as Uint8List);
    var topLevelSeq = asn1Parser.nextObject() as ASN1Sequence;

    var modulus, privateExponent, p, q;
    //Use either PKCS1 or PKCS8 depending on the number of ELEMENTS
    if (topLevelSeq.elements.length == 3) {
      var privateKey = topLevelSeq.elements[2];

      asn1Parser = new ASN1Parser(privateKey.contentBytes()!);
      var pkSeq = asn1Parser.nextObject() as ASN1Sequence;

      modulus = pkSeq.elements[1] as ASN1Integer;
      privateExponent = pkSeq.elements[3] as ASN1Integer;
      p = pkSeq.elements[4] as ASN1Integer;
      q = pkSeq.elements[5] as ASN1Integer;
    } else {
      modulus = topLevelSeq.elements[1] as ASN1Integer;
      privateExponent = topLevelSeq.elements[3] as ASN1Integer;
      p = topLevelSeq.elements[4] as ASN1Integer;
      q = topLevelSeq.elements[5] as ASN1Integer;
    }

    RSAPrivateKey rsaPrivateKey = RSAPrivateKey(
        modulus.valueAsBigInteger,
        privateExponent.valueAsBigInteger,
        p.valueAsBigInteger,
        q.valueAsBigInteger);

    return rsaPrivateKey;
  }

  Future<String> _encryptData(String paymentTransactionJson)async{

    final encrypter = encry.Encrypter(
        encry.RSA(encoding: encry.RSAEncoding.PKCS1,
          publicKey: _parsePublicKeyFromPem(_publicKeyString),
          // privateKey: parsePrivateKeyFromPem(privateKeyString)
        )
    );
    final encrypted = encrypter.encrypt(paymentTransactionJson);
    // final decrypted = encrypter.decrypt(encrypted);
    // print(encrypted.base64);
    return encrypted.base64;
  }

Future<ServerResponse> pay({required PaymentTransaction paymentTransaction, required BuildContext context})async{
  String jsonString = json.encode(paymentTransaction.toJson());
  String encryptedData = await _encryptData(jsonString);
  ServerResponse result = ServerResponse(message: "Transaction Error", status: "ERROR");
  if(_companyId.isEmpty || _publicKeyString.isEmpty){
    const snackBar = SnackBar(
        backgroundColor: Colors.white,
        content: Text(
          'Getit Payments Not Initialized.',
          style: TextStyle(color: Colors.black),
        ));
    ScaffoldMessenger.of(context).showSnackBar(snackBar);
    return result;
  }
  Map auth = {
    'testMode': _testMode,
    'companyId': _companyId,
    'encryptedTxt': encryptedData,
    // 'otherData': ecryOtherData
  };
  Uri url = Uri.parse('https://us-central1-getitpayments.cloudfunctions.net/makPayment');
  try {
    await http.post(url, body: auth).then((response) async {
      var data = json.decode(response.body);
      // debugPrint("data : "+data.toString());
      if (data != null) {
        if (data["msg"] is String) {
          result.message = data["msg"];
        } else {
          if (data["msg"]["ResultCode"] != null) {
            result.status = data["msg"]["ResultCode"][0];
          }
          if (data["msg"]["FinancialResultCode"] != null) {
            result.message = data["msg"]["FinancialResultCode"][0]
                .toString(); // if it says Approved then you good.
          }

          if (data["msg"]["ErrorMessage"] != null) {
            result.error = data["msg"]["ErrorMessage"][0].toString();
          }
          if (data["msg"]["TranId"] != null) {
            result.transactionNumber = data["msg"]["TranId"][0].toString();
          }
          if(data["msg"]["Invoice"] != null){
            result.invoiceNumber = data["msg"]["Invoice"][0].toString();
          }
        }
      }
    });
  } catch (e) {
    String error = "`pay_function.dart` ${e.toString()}";
    // FirebaseCrashlytics.instance.log(error);
    log(error);
  }
  return result;
}

}
